"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadFirmwareUpdate = exports.getAvailableFirmwareUpdates = void 0;
const got_1 = __importDefault(require("@esm2cjs/got"));
const p_queue_1 = __importDefault(require("@esm2cjs/p-queue"));
const core_1 = require("@zwave-js/core");
const shared_1 = require("@zwave-js/shared");
const crypto_1 = __importDefault(require("crypto"));
function serviceURL() {
    return process.env.ZWAVEJS_FW_SERVICE_URL || "https://firmware.zwave-js.io";
}
const DOWNLOAD_TIMEOUT = 60000;
// const MAX_FIRMWARE_SIZE = 10 * 1024 * 1024; // 10MB should be enough for any conceivable Z-Wave chip
const requestCache = new Map();
// Queue requests to the firmware update service. Only allow few parallel requests so we can make some use of the cache.
const requestQueue = new p_queue_1.default({ concurrency: 2 });
/** Converts the RF region to a format the update service understands */
function rfRegionToUpdateServiceRegion(rfRegion) {
    switch (rfRegion) {
        case core_1.RFRegion["Default (EU)"]:
        case core_1.RFRegion.Europe:
            return "europe";
        case core_1.RFRegion.USA:
        case core_1.RFRegion["USA (Long Range)"]:
            return "usa";
        case core_1.RFRegion["Australia/New Zealand"]:
            return "australia/new zealand";
        case core_1.RFRegion["Hong Kong"]:
            return "hong kong";
        case core_1.RFRegion.India:
            return "india";
        case core_1.RFRegion.Israel:
            return "israel";
        case core_1.RFRegion.Russia:
            return "russia";
        case core_1.RFRegion.China:
            return "china";
        case core_1.RFRegion.Japan:
            return "japan";
        case core_1.RFRegion.Korea:
            return "korea";
    }
}
/**
 * Retrieves the available firmware updates for the node with the given fingerprint.
 * Returns the service response or `undefined` in case of an error.
 */
function getAvailableFirmwareUpdates(deviceId, options) {
    const headers = {
        "User-Agent": options.userAgent,
        "Content-Type": "application/json",
    };
    if (options.apiKey) {
        headers["X-API-Key"] = options.apiKey;
    }
    const body = {
        manufacturerId: (0, shared_1.formatId)(deviceId.manufacturerId),
        productType: (0, shared_1.formatId)(deviceId.productType),
        productId: (0, shared_1.formatId)(deviceId.productId),
        firmwareVersion: deviceId.firmwareVersion,
    };
    const rfRegion = rfRegionToUpdateServiceRegion(deviceId.rfRegion);
    if (rfRegion) {
        body.region = rfRegion;
    }
    const config = {
        method: "POST",
        url: `${serviceURL()}/api/${options.includePrereleases ? "v3" : "v1"}/updates`,
        json: body,
        cache: requestCache,
        cacheOptions: {
            shared: false,
        },
        headers,
    };
    return requestQueue.add(() => {
        return (0, got_1.default)(config).json();
    });
}
exports.getAvailableFirmwareUpdates = getAvailableFirmwareUpdates;
async function downloadFirmwareUpdate(file) {
    const [hashAlgorithm, expectedHash] = file.integrity.split(":", 2);
    if (hashAlgorithm !== "sha256") {
        throw new core_1.ZWaveError(`Unsupported hash algorithm ${hashAlgorithm} for integrity check`, core_1.ZWaveErrorCodes.Argument_Invalid);
    }
    // TODO: Make request abort-able (requires AbortController, Node 14.17+ / Node 16)
    // Download the firmware file
    const downloadResponse = await got_1.default.get(file.url, {
        timeout: { request: DOWNLOAD_TIMEOUT },
        responseType: "buffer",
        // TODO: figure out how to do maxContentLength: MAX_FIRMWARE_SIZE,
    });
    const rawData = downloadResponse.body;
    // Infer the file type from the content-disposition header or the filename
    let filename;
    if (downloadResponse.headers["content-disposition"]?.startsWith("attachment; filename=")) {
        filename = downloadResponse.headers["content-disposition"]
            .split("filename=")[1]
            .replace(/^"/, "")
            .replace(/[";]$/, "");
    }
    else {
        filename = new URL(file.url).pathname;
    }
    // Extract the raw data
    const format = (0, core_1.guessFirmwareFileFormat)(filename, rawData);
    const firmware = (0, core_1.extractFirmware)(rawData, format);
    // Ensure the hash matches
    const hasher = crypto_1.default.createHash("sha256");
    hasher.update(firmware.data);
    const actualHash = hasher.digest("hex");
    if (actualHash !== expectedHash) {
        throw new core_1.ZWaveError(`Integrity check failed. Expected hash ${expectedHash}, got ${actualHash}`, core_1.ZWaveErrorCodes.FWUpdateService_IntegrityCheckFailed);
    }
    return {
        data: firmware.data,
        // Don't trust the guessed firmware target, use the one from the provided info
        firmwareTarget: file.target,
    };
}
exports.downloadFirmwareUpdate = downloadFirmwareUpdate;
//# sourceMappingURL=FirmwareUpdateService.js.map